/*
 * This file contains "prototypes" for functions used by Icon. These are
 *  processed after system-dependent symbols and typedefs.
 *
 * There are two major sections: one for C compilers that accept full
 *  prototypes with arguments and one for C compilers that do not. The
 *  choice of section depends on whether ProtoArgs is defined.  Its
 *  definition would occur in define.h.
 *
 * Within the major sections, prototypes are divided into three groups:
 *
 *	library functions
 *	functions in icont
 *	functions in iconx
 *
 * Notice that within groups, the functions are in alphabetical order
 *  except for a few special cases. This is merely to make it easier to
 *  maintain this file.
 *
 * Prototypes for some library functions are deliberately omitted, since
 *  they vary from compiler to compiler. These include those for free();
 *  sbrk(), and lsbrk().
 */

#ifdef ProtoArgs
/*
 * Prototypes with arguments.
 */

/*
 * Prototypes for library functions.
 */
/*extern*/ novalue _exit(int);
/*extern*/ novalue abort(void);
/*extern*/ long atol(char *);
/*extern*/ pointer calloc(unsigned,unsigned);
/*extern*/ int execv(char *, char **);
/*extern*/ int execvp(char *, char **);
/*extern*/ novalue exit(int);
/*extern*/ int free(char *);
/*extern*/ char *getenv(char *);
/*extern*/ char *getmem(unsigned);
/*extern*/ pointer malloc(unsigned);
/*extern*/ novalue qsort(char *, int, int, int (*)(void));
/*extern*/ pointer realloc(char*, unsigned);
/*extern*/ int system(char *);
/*extern*/ int wait(void);

/*
 * Prototypes for functions in icont.
 */
/*extern*/ novalue addlfile(char *name);
/*static*/ struct fentry *alcfhead(struct fentry *blink,char *name,int fid,struct rentry *rlist);
/*static*/ struct rentry *alcfrec(struct rentry *link,int rnum,int fnum);
/*static*/ struct tgentry *alcglob(struct tgentry *blink,char *name,int flag,int nargs);
/*static*/ struct gentry *alcglobal(struct gentry *blink,char *name,int flag,int nargs,int procid);
/*static*/ struct tientry *alcid(char *nam,int len);
/*static*/ struct ientry *alcident(char *nam,int len);
/*static*/ int alclab(int n);
/*extern*/ struct lfile *alclfile(char *name);
/*static*/ struct tcentry *alclit(struct tcentry *blink,char *name,int len,int flag);
/*static*/ struct tlentry *alcloc(struct tlentry *blink,char *name,int flag);
/*extern*/ pointer alloc(unsigned int n);
/*extern*/ novalue alsolink(char *name);
/*static*/ novalue backpatch(int lab);
/*static*/ novalue binop(int op);
/*static*/ int blocate(char *s);
/*extern*/ struct node *c_str_leaf(int type,struct node *loc_model,char *c);
/*static*/ int canread(char *file);
/*static*/ novalue cdump(char *);
/*static*/ novalue clearlab(void);
/*static*/ struct tcentry *clookup(char *id,int flag);
/*extern*/ novalue codegen(struct node *t);
/*extern*/ novalue cout(FILE *fd);
/*static*/ int ctlesc(novalue);
/*extern*/ novalue dummyda(void);
/*static*/ novalue dumpblock(char *addr,int count);
/*static*/ novalue emit(char *s);
/*static*/ novalue emitl(char *s,int a);
/*static*/ novalue emitlab(int l);
/*static*/ novalue emitn(char *s,int a);
/*static*/ novalue emits(char *s,char *a);
/*static*/ novalue execute(char *ofile,char *efile,char * *args);
/*static*/ struct toktab *findres(novalue);
/*extern*/ struct fentry *flocate(char *id);
/*static*/ novalue flushcode(void);
/*extern*/ struct fileparts fparse(char *s);
/*extern*/ novalue gencode(void);
/*extern*/ novalue gentables(void);
/*extern*/ int getdec(novalue);
/*extern*/ char *getid(novalue);
/*static*/ struct toktab *getident(char ac,int *cc);
/*extern*/ long getint(novalue);
/*extern*/ int getlab(novalue);
/*extern*/ struct lfile *getlfile(struct lfile * *lptr);
/*static*/ struct toktab *getnum(char ac,int *cc);
/*extern*/ int getoct(novalue);
/*extern*/ int getopc(char * *id);
/*static*/ struct toktab *getopr(char ac,int *cc);
/*static*/ int getopt(int nargc,char * *nargv,char *ostr);
/*extern*/ double getreal(novalue);
/*extern*/ char *getstr(novalue);
/*static*/ struct toktab *getstring(char ac,int *cc);
/*extern*/ char *getstrlit(int l);
/*extern*/ struct gentry *glocate(char *id);
/*static*/ struct tgentry *glookup(char *id);
/*extern*/ novalue gout(FILE *fd);
/*static*/ int hexesc(novalue);
/*extern*/ struct node *i_str_leaf(int type,struct node *loc_model,char *c,int d);
/*extern*/ int ilink(char * *ifiles,char *outname);
/*extern*/ novalue install(char *name,int flag,int argcnt);
/*extern*/ char *instid(char *s);
/*static*/ unsigned int round2(unsigned int n);
/*extern*/ struct node *int_leaf(int type,struct node *loc_model,int c);
/*static*/ novalue intout(int oint);
/*extern*/ int klookup(char *id);
/*static*/ novalue lemit(int op,char *name);
/*static*/ novalue lemitcon(int k);
/*static*/ novalue lemiteven(void);
/*static*/ novalue lemitin(int op,long offset,int n,char *name);
/*static*/ novalue lemitint(int op,long i,char *name);
/*static*/ novalue lemitl(int op,int lab,char *name);
/*static*/ novalue lemitn(int op,long n,char *name);
/*static*/ novalue lemitproc(char *name,int nargs,int ndyn,int nstat,int fstat);
/*static*/ novalue lemitr(int op,long loc,char *name);
/*extern*/ int lexeql(int l,char *s1,char *s2);
/*extern*/ novalue lfatal(char *s1,char *s2);
/*extern*/ novalue linit(void);
/*static*/ struct tlentry *llookup(char *id);
/*extern*/ novalue lmfree(void);
/*extern*/ novalue loc_init(void);
/*extern*/ novalue locinit(void);
/*extern*/ novalue longwrite(char *s,long len,FILE *file);
/*extern*/ novalue lout(FILE *fd);
/*extern*/ novalue lwarn(char *s1,char *s2,char *s3);
/*extern*/ char *makename(char *dest,char *d,char *name,char *e);
/*static*/ char *mapterm(int typ,struct node *val);
/*extern*/ novalue newline(void);
/*static*/ int nextchar(novalue);
/*extern*/ novalue nfatal(struct node *n,char *s);
/*static*/ int octesc(char ac);
/*static*/ novalue outblock(char *addr,int count);
/*static*/ FILE *preprocess(char *filename);
/* novalue putconst(int n,int flags,int len,long pc,union UNNAMED *valp); */
/*extern*/ novalue putfield(char *fname,int rnum,int fnum);
/*static*/ novalue putglob(char *id,int id_type,int n_args);
/*extern*/ struct gentry *putglobal(char *id,int flags,int nargs,int procid);
/*extern*/ char *putid(int len);
/*extern*/ char *putident(int len);
/*extern*/ int putlit(char *id,int idtype,int len);
/*extern*/ int putloc(char *id,int id_type);
/*extern*/ novalue putlocal(int n,char *id,int flags,int imperror,char *procname);
/*extern*/ novalue quit(char *msg);
/*extern*/ novalue quitf(char *msg,char *arg);
/*extern*/ novalue readglob(void);
/*static*/ novalue report(char *s);
/*static*/ novalue rmfiles(char * *p);
/*extern*/ novalue rout(FILE *fd,char *name);
/*extern*/ char *salloc(char *s);
/*static*/ novalue setexe(char *fname);
/*static*/ int setfilenm(int c);
/*static*/ int setlineno(novalue);
/*static*/ novalue setloc(struct node *n);
/*static*/ novalue sizearg(char *arg,char * *argv);
/*static*/ int smatch(char *s,char *t);
/*static*/ int streq(int len,char *s1,char *s2);
/*extern*/ pointer tcalloc(unsigned int m,unsigned int n);
/*extern*/ novalue tfatal(char *s1,char *s2);
/*extern*/ novalue tmalloc(void);
/*extern*/ novalue tmfree(void);
/*extern*/ novalue tminit(void);
/*extern*/ int trans(char * *ifiles);
/*static*/ novalue trans1(char *filename);
/*static*/ int traverse(struct node *t);
/*extern*/ struct node *tree1(int type);
/*extern*/ struct node *tree2(int type,struct node *loc_model);
/*extern*/ struct node *tree3(int type,struct node *loc_model,struct node *c);
/*extern*/ struct node *tree4(int type,struct node *loc_model,struct node *c,struct node *d);
/*extern*/ struct node *tree5(int type,struct node *loc_model,struct node *c,struct node *d,struct node *e);
/*extern*/ struct node *tree6(int type,struct node *loc_model,struct node *c,struct node *d,struct node *e,struct node *f);
/*extern*/ novalue treeinit(void);
/*static*/ int trypath(char *name,char *file);
/*extern*/ novalue tsyserr(char *s);
/*extern*/ novalue twarn(char *s1,char *s2);
/*static*/ novalue unopa(int op);
/*static*/ novalue unopb(int op);
/*static*/ novalue usage(void);
/*static*/ novalue wordout(long oword);
/*extern*/ novalue writecheck(int rc);
/*extern*/ novalue yyerror(int tok,struct node *lval,int state);
/*extern*/ int yylex(novalue);
/*extern*/ int yyparse(novalue);

/*
 * Prototypes for functions in iconx.
 */
/*extern*/ int Xabs(struct descrip *cargp);
/*extern*/ int Xany(struct descrip *cargp);
/*extern*/ int Xbal(struct descrip *cargp);
/*extern*/ int Xcenter(struct descrip *cargp);
/*extern*/ int Xchar(struct descrip *cargp);
/*extern*/ int Xclose(struct descrip *cargp);
/*extern*/ int Xcollect(struct descrip *cargp);
/*extern*/ int Xcopy(struct descrip *cargp);
/*extern*/ int Xcset(struct descrip *cargp);
/*extern*/ int Xdelete(struct descrip *cargp);
/*extern*/ int Xdetab(int nargs,struct descrip *cargp);
/*extern*/ int Xdisplay(struct descrip *cargp);
/*extern*/ int Xentab(int nargs,struct descrip *cargp);
/*extern*/ int Xerrorclear(struct descrip *cargp);
/*extern*/ int Xexit(struct descrip *cargp);
/*extern*/ int Xfind(struct descrip *cargp);
/*extern*/ int Xget(struct descrip *cargp);
/*extern*/ int Xgetenv(struct descrip *cargp);
/*extern*/ int Xiand(struct descrip *cargp);
/*extern*/ int Xicom(struct descrip *cargp);
/*extern*/ int Ximage(struct descrip *cargp);
/*extern*/ int Xinsert(struct descrip *cargp);
/*extern*/ int Xinteger(struct descrip *cargp);
/*extern*/ int Xior(struct descrip *cargp);
/*extern*/ int Xishift(struct descrip *cargp);
/*extern*/ int Xixor(struct descrip *cargp);
/*extern*/ int Xleft(struct descrip *cargp);
/*extern*/ int Xlist(struct descrip *cargp);
/*extern*/ int Xmany(struct descrip *cargp);
/*extern*/ int Xmap(struct descrip *cargp);
/*extern*/ int Xmatch(struct descrip *cargp);
/*extern*/ int Xmember(struct descrip *cargp);
/*extern*/ int Xmove(struct descrip *cargp);
/*extern*/ int Xnumeric(struct descrip *cargp);
/*extern*/ int Xopen(struct descrip *cargp);
/*extern*/ int Xord(struct descrip *cargp);
/*extern*/ int Xpop(struct descrip *cargp);
/*extern*/ int Xpos(struct descrip *cargp);
/*extern*/ int Xproc(struct descrip *cargp);
/*extern*/ int Xpull(struct descrip *cargp);
/*extern*/ int Xpush(struct descrip *cargp);
/*extern*/ int Xput(struct descrip *cargp);
/*extern*/ int Xread(struct descrip *cargp);
/*extern*/ int Xreads(struct descrip *cargp);
/*extern*/ int Xreal(struct descrip *cargp);
/*extern*/ int Xremove(struct descrip *cargp);
/*extern*/ int Xrename(struct descrip *cargp);
/*extern*/ int Xrepl(struct descrip *cargp);
/*extern*/ int Xreverse(struct descrip *cargp);
/*extern*/ int Xright(struct descrip *cargp);
/*extern*/ int Xrunerr(int nargs,struct descrip *cargp);
/*extern*/ int Xseek(struct descrip *cargp);
/*extern*/ int Xseq(struct descrip *cargp);
/*extern*/ int Xset(struct descrip *cargp);
/*extern*/ int Xsort(struct descrip *cargp);
/*extern*/ int Xstop(int nargs,struct descrip *cargp);
/*extern*/ int Xstring(struct descrip *cargp);
/*extern*/ int Xsystem(struct descrip *cargp);
/*extern*/ int Xtab(struct descrip *cargp);
/*extern*/ int Xtable(struct descrip *cargp);
/*extern*/ int Xtrim(struct descrip *cargp);
/*extern*/ int Xtype(struct descrip *cargp);
/*extern*/ int Xupto(struct descrip *cargp);
/*extern*/ int Xwhere(struct descrip *cargp);
/*extern*/ int Xwrite(int nargs,struct descrip *cargp);
/*extern*/ int Xwrites(int nargs,struct descrip *cargp);
/*extern*/ int abs(int);
/*extern*/ novalue addmem(struct b_set *ps,struct b_selem *pe,struct descrip *pl);
/*extern*/ novalue adjust(char *source,char *dest);
/*extern*/ struct astkblk *alcactiv(void);
/*extern*/ struct b_coexpr *alccoexp(void);
/*extern*/ struct b_cset *alccset(void);
/*extern*/ struct b_external *alcextrnl(int n);
/*extern*/ struct b_file *alcfile(FILE *fd,int status,struct descrip *name);
/*extern*/ struct b_list *alclist(unsigned long size);
/*extern*/ struct b_lelem *alclstb(unsigned long nslots,unsigned long first,unsigned long nused);
/*extern*/ struct b_real *alcreal(double val);
/*extern*/ struct b_record *alcrecd(int nflds,struct descrip *recptr);
/*extern*/ struct b_refresh *alcrefresh(long *entryx,int na,int nl);
/*extern*/ struct b_selem *alcselem(struct descrip *mbr,long hn);
/*extern*/ struct b_set *alcset(void);
/*extern*/ char *alcstr(char *s,long slen);
/*extern*/ struct b_tvsubs *alcsubs(long len,long pos,struct descrip *var);
/*extern*/ struct b_table *alctable(struct descrip *def);
/*extern*/ struct b_telem *alctelem(void);
/*extern*/ struct b_tvtbl *alctvtbl(struct descrip *tbl,struct descrip *ref,long hashnum);
/*extern*/ int anycmp(struct descrip *dp1,struct descrip *dp2);
/*extern*/ int asgn(struct descrip *cargp);
/*extern*/ novalue atrace(struct b_proc *bp);
/*extern*/ int bang(struct descrip *cargp);
/*extern*/ int blkreq(unsigned long bytes);
/*static*/ union block *alcblk(unsigned long nbytes,int tcode);
/*extern*/ int bscan(int nargs,struct descrip *cargp);
/*extern*/ novalue c_exit(int i);
/*extern*/ int cat(struct descrip *cargp);
/*extern*/ novalue coacttrace(struct b_coexpr *ccp,struct b_coexpr *ncp);
/*extern*/ novalue cofailtrace(struct b_coexpr *ccp,struct b_coexpr *ncp);
/*extern*/ novalue cofree(void);
/*extern*/ novalue collect(int region);
/*extern*/ novalue compact(char *source);
/*extern*/ int compl(struct descrip *cargp);
/*extern*/ novalue corettrace(struct b_coexpr *ccp,struct b_coexpr *ncp);
/*extern*/ int cplist(struct descrip *dp1,struct descrip *dp2,long i,long j);
/*extern*/ int create(long *entryp,struct descrip *cargp);
/*extern*/ int csetcmp(unsigned int *cs1,unsigned int *cs2);
/*static*/ int cstos(int *cs,struct descrip *dp,char *s);
/*extern*/ novalue ctrace(struct b_proc *bp,int nargs,struct descrip *arg);
/*extern*/ int cvcset(struct descrip *dp,int * *cs,int *csbuf);
/*extern*/ int cvint(struct descrip *dp,long *i);
/*extern*/ int cvnum(struct descrip *dp,union numeric *result);
/*extern*/ long cvpos(long pos,long len);
/*extern*/ int cvreal(struct descrip *dp,double *r);
/*extern*/ int cvstr(struct descrip *dp,char *sbuf);
/*extern*/ novalue datainit(void);
/*extern*/ int defcset(struct descrip *dp,int * *cp,int *buf,int *def);
/*extern*/ int deffile(struct descrip *dp,struct descrip *def);
/*extern*/ int defint(struct descrip *dp,long *lp,long def);
/*extern*/ int defshort(struct descrip *dp,int def);
/*extern*/ int defstr(struct descrip *dp,char *buf,struct descrip *def);
/*extern*/ int deref(struct descrip *dp);
/*extern*/ int diff(struct descrip *cargp);
/*extern*/ int divide(struct descrip *cargp);
/*extern*/ int doasgn(struct descrip *dp1,struct descrip *dp2);
/*extern*/ int doimage(int c,int q);
/*static*/ novalue env_err(char *msg,char *name,char *val);
/*extern*/ novalue env_int(char *name,long *variable,int non_neg);
/*extern*/ int envset(void);
/*extern*/ int equiv(struct descrip *dp1,struct descrip *dp2);
/*extern*/ int eqv(struct descrip *cargp);
/*extern*/ int err(void);
/*extern*/ novalue error(char *s);
/*extern*/ int escan(int nargs,struct descrip *cargp);
/*extern*/ novalue fatalerr(int n,struct descrip *v);
/*extern*/ int field(int nargs,struct descrip *cargp);
/*extern*/ char *findfile(long *ipc);
/*extern*/ int findline(long *ipc);
/*extern*/ int fpetrap(void);
/*extern*/ novalue ftrace(struct b_proc *bp);
/*extern*/ int getstrg(char *buf,int maxi,FILE *fd);
/*extern*/ long hash(struct descrip *dp);
/*extern*/ char *icon_gcvt(double number,int ndigit,char *buf);
/*extern*/ novalue iconhost(char *hostname);
/*extern*/ novalue init(char *name);
/*extern*/ novalue initalloc(long codesize);
/*extern*/ int inter(struct descrip *cargp);
/*extern*/ int interp(int fsig,struct descrip *cargp);
/*extern*/ int invoke(int nargs,struct descrip * *cargp,int *n);
/*extern*/ long ipow(long n1,long n2);
/*static*/ int itos(long num,struct descrip *dp,char *s);
/*extern*/ int keywd(int nargs,struct descrip *cargp);
/*extern*/ int lconcat(struct descrip *cargp);
/*extern*/ int lexcmp(struct descrip *dp1,struct descrip *dp2);
/*extern*/ int lexeq(struct descrip *cargp);
/*extern*/ int lexge(struct descrip *cargp);
/*extern*/ int lexgt(struct descrip *cargp);
/*extern*/ int lexle(struct descrip *cargp);
/*extern*/ int lexlt(struct descrip *cargp);
/*extern*/ int lexne(struct descrip *cargp);
/*extern*/ int limit(int nargs,struct descrip *cargp);
/*static*/ novalue listimage(FILE *f,struct b_list *lp,int restrict);
/*extern*/ int llist(int nargs,struct descrip *cargp);
/*extern*/ int locate(struct b_selem *ep,struct b_selem *obj);
/*extern*/ long longread(char *s,int width,long len,FILE *fname);
/*extern*/ long lstrlen(char *s);
/*extern*/ novalue markblock(struct descrip *dp);
/*extern*/ struct descrip *memb(struct b_set *ps,struct descrip *x,long hn,int *res);
/*extern*/ struct descrip *memb_tbl(struct b_table *pt,struct descrip *x,long hn,int *res);
/*extern*/ int minus(struct descrip *cargp);
/*extern*/ int mkreal(double r,struct descrip *dp);
/*extern*/ int mkrec(int nargs,struct descrip *cargp);
/*extern*/ novalue mksubs(struct descrip *var,struct descrip *val,long i,long j,struct descrip *result);
/*extern*/ int mod(struct descrip *cargp);
/*extern*/ int mult(struct descrip *cargp);
/*extern*/ novalue mvc(unsigned long n,char *src,char *dest);
/*extern*/ int neg(struct descrip *cargp);
/*extern*/ int neqv(struct descrip *cargp);
/*extern*/ int nonnull(struct descrip *cargp);
/*extern*/ int null(struct descrip *cargp);
/*extern*/ int number(struct descrip *cargp);
/*extern*/ int numcmp(struct descrip *dp1,struct descrip *dp2,struct descrip *dp3);
/*extern*/ int numeq(struct descrip *cargp);
/*extern*/ int numge(struct descrip *cargp);
/*extern*/ int numgt(struct descrip *cargp);
/*extern*/ int numle(struct descrip *cargp);
/*extern*/ int numlt(struct descrip *cargp);
/*extern*/ int numne(struct descrip *cargp);
/*static*/ int nxttab(int col);
/*extern*/ int order(struct descrip *dp);
/*extern*/ novalue outimage(FILE *f,struct descrip *dp,int restrict);
/*extern*/ int plus(struct descrip *cargp);
/*extern*/ struct b_coexpr *popact(struct b_coexpr *ce);
/*extern*/ novalue postqual(struct descrip *dp);
/*extern*/ int power(struct descrip *cargp);
/*extern*/ long prescan(struct descrip *d);
/*static*/ novalue printimage(FILE *f,int c,int q);
/*extern*/ int pushact(struct b_coexpr *ce,struct b_coexpr *actvtr);
/*extern*/ int puterr(struct descrip *dp);
/*extern*/ int putpos(struct descrip *dp);
/*extern*/ int putran(struct descrip *dp);
/*extern*/ int putstr(FILE *f,struct descrip *d);
/*extern*/ int putsub(struct descrip *dp);
/*extern*/ int puttrc(struct descrip *dp);
/*extern*/ int qlcmp(struct descrip * *q1,struct descrip * *q2);
/*extern*/ int qtos(struct descrip *dp,char *sbuf);
/*static*/ int radix(char sign,int r,char *s,union numeric *result);
/*extern*/ int random(struct descrip *cargp);
/*extern*/ int rasgn(struct descrip *cargp);
/*static*/ novalue reclaim(int region);
/*extern*/ int refresh(struct descrip *cargp);
/*extern*/ int resolve(void);
/*extern*/ int rswap(struct descrip *cargp);
/*extern*/ int rtos(double n,struct descrip *dp,char *s);
/*extern*/ novalue rtrace(struct b_proc *bp,struct descrip *rval);
/*extern*/ novalue runerr(int n,struct descrip *v);
/*extern*/ novalue scollect(long extra);
/*extern*/ int sect(struct descrip *cargp);
/*extern*/ int segvtrap(void);
/*static*/ novalue showlevel(int n);
/*static*/ novalue showline(char *f,int l);
/*extern*/ int size(struct descrip *cargp);
/*static*/ novalue stkdump(int);
/*static*/ int ston(char *s,union numeric *result);
/*extern*/ novalue strace(struct b_proc *bp,struct descrip *rval);
/*extern*/ int strprc(struct descrip *dp,long n);
/*extern*/ int strreq(unsigned long n);
/*extern*/ int subsc(struct descrip *cargp);
/*extern*/ int swap(struct descrip *cargp);
/*extern*/ novalue sweep(struct b_coexpr *ce);
/*extern*/ novalue syserr(char *s);
/*extern*/ int tabmat(struct descrip *cargp);
/*extern*/ int toby(struct descrip *cargp);
/*extern*/ struct b_coexpr *topact(struct b_coexpr *ce);
/*extern*/ int trcmp3(struct dpair *dp1,struct dpair *dp2);
/*extern*/ int trefcmp(struct descrip *d1,struct descrip *d2);
/*extern*/ novalue ttrace(void);
/*extern*/ int tvalcmp(struct descrip *d1,struct descrip *d2);
/*extern*/ int tvcmp4(struct dpair *dp1,struct dpair *dp2);
/*extern*/ int unions(struct descrip *cargp);
/*extern*/ int value(struct descrip *cargp);
/*extern*/ novalue xdisp(int count,FILE *f);
/*extern*/ novalue xtrace(struct b_proc *bp,long nargs,struct descrip *arg,int pline,char *pfile);

#else					/* ProtoArgs */

/*
 * Prototypes without arguments.
 */

/*
 * Prototypes for library functions.
 */
/*extern*/ novalue _exit();
/*extern*/ novalue abort();
/*extern*/ long atol();
/*extern*/ pointer calloc();
/*extern*/ int execv();
/*extern*/ int execvp();
/*extern*/ novalue exit();
/*extern*/ int free();
/*extern*/ char *getenv();
/*extern*/ char *getmem();
/*extern*/ pointer malloc();
/*extern*/ novalue qsort();
/*extern*/ pointer realloc();
/*extern*/ int system();
/*extern*/ int wait();

/*
 * Prototypes for functions in icont.
 */
/*extern*/ novalue addlfile();
/*static*/ struct fentry *alcfhead();
/*static*/ struct rentry *alcfrec();
/*static*/ struct tgentry *alcglob();
/*static*/ struct gentry *alcglobal();
/*static*/ struct tientry *alcid();
/*static*/ struct ientry *alcident();
/*static*/ int alclab();
/*extern*/ struct lfile *alclfile();
/*static*/ struct tcentry *alclit();
/*static*/ struct tlentry *alcloc();
/*extern*/ pointer alloc();
/*extern*/ novalue alsolink();
/*static*/ novalue backpatch();
/*static*/ novalue binop();
/*static*/ int blocate();
/*extern*/ struct node *c_str_leaf();
/*static*/ int canread();
/*static*/ novalue cdump();
/*static*/ novalue clearlab();
/*static*/ struct tcentry *clookup();
/*extern*/ novalue codegen();
/*extern*/ novalue cout();
/*static*/ int ctlesc();
/*extern*/ novalue dummyda();
/*static*/ novalue dumpblock();
/*static*/ novalue emit();
/*static*/ novalue emitl();
/*static*/ novalue emitlab();
/*static*/ novalue emitn();
/*static*/ novalue emits();
/*static*/ novalue execute();
/*static*/ struct toktab *findres();
/*extern*/ struct fentry *flocate();
/*static*/ novalue flushcode();
/*extern*/ struct fileparts fparse();
/*extern*/ novalue gencode();
/*extern*/ novalue gentables();
/*extern*/ int getdec();
/*extern*/ char *getid();
/*static*/ struct toktab *getident();
/*extern*/ long getint();
/*extern*/ int getlab();
/*extern*/ struct lfile *getlfile();
/*static*/ struct toktab *getnum();
/*extern*/ int getoct();
/*extern*/ int getopc();
/*static*/ struct toktab *getopr();
/*static*/ int getopt();
/*extern*/ double getreal();
/*extern*/ char *getstr();
/*static*/ struct toktab *getstring();
/*extern*/ char *getstrlit();
/*extern*/ struct gentry *glocate();
/*static*/ struct tgentry *glookup();
/*extern*/ novalue gout();
/*static*/ int hexesc();
/*extern*/ struct node *i_str_leaf();
/*extern*/ int ilink();
/*extern*/ novalue install();
/*extern*/ char *instid();
/*static*/ unsigned int round2();
/*extern*/ struct node *int_leaf();
/*static*/ novalue intout();
/*extern*/ int klookup();
/*static*/ novalue lemit();
/*static*/ novalue lemitcon();
/*static*/ novalue lemiteven();
/*static*/ novalue lemitin();
/*static*/ novalue lemitint();
/*static*/ novalue lemitl();
/*static*/ novalue lemitn();
/*static*/ novalue lemitproc();
/*static*/ novalue lemitr();
/*extern*/ int lexeql();
/*extern*/ novalue lfatal();
/*extern*/ novalue linit();
/*static*/ struct tlentry *llookup();
/*extern*/ novalue lmfree();
/*extern*/ novalue loc_init();
/*extern*/ novalue locinit();
/*extern*/ novalue longwrite();
/*extern*/ novalue lout();
/*extern*/ novalue lwarn();
/*extern*/ char *makename();
/*static*/ char *mapterm();
/*extern*/ novalue newline();
/*static*/ int nextchar();
/*extern*/ novalue nfatal();
/*static*/ int octesc();
/*static*/ novalue outblock();
/*static*/ FILE *preprocess();
/*extern*/ novalue putconst();
/*extern*/ novalue putfield();
/*static*/ novalue putglob();
/*extern*/ struct gentry *putglobal();
/*extern*/ char *putid();
/*extern*/ char *putident();
/*extern*/ int putlit();
/*extern*/ int putloc();
/*extern*/ novalue putlocal();
/*extern*/ novalue quit();
/*extern*/ novalue quitf();
/*extern*/ novalue readglob();
/*static*/ novalue report();
/*static*/ novalue rmfiles();
/*extern*/ novalue rout();
/*extern*/ char *salloc();
/*static*/ novalue setexe();
/*static*/ int setfilenm();
/*static*/ int setlineno();
/*static*/ novalue setloc();
/*static*/ novalue sizearg();
/*static*/ int smatch();
/*static*/ int streq();
/*extern*/ pointer tcalloc();
/*extern*/ novalue tfatal();
/*extern*/ novalue tmalloc();
/*extern*/ novalue tmfree();
/*extern*/ novalue tminit();
/*extern*/ int trans();
/*static*/ novalue trans1();
/*static*/ int traverse();
/*extern*/ struct node *tree1();
/*extern*/ struct node *tree2();
/*extern*/ struct node *tree3();
/*extern*/ struct node *tree4();
/*extern*/ struct node *tree5();
/*extern*/ struct node *tree6();
/*extern*/ novalue treeinit();
/*static*/ int trypath();
/*extern*/ novalue tsyserr();
/*extern*/ novalue twarn();
/*static*/ novalue unopa();
/*static*/ novalue unopb();
/*static*/ novalue usage();
/*static*/ novalue wordout();
/*extern*/ novalue writecheck();
/*extern*/ novalue yyerror();
/*extern*/ int yylex();
/*extern*/ int yyparse();

/*
 * Prototypes for functions in iconx.
 */
/*extern*/ int Xabs();
/*extern*/ int Xany();
/*extern*/ int Xbal();
/*extern*/ int Xcenter();
/*extern*/ int Xchar();
/*extern*/ int Xclose();
/*extern*/ int Xcollect();
/*extern*/ int Xcopy();
/*extern*/ int Xcset();
/*extern*/ int Xdelete();
/*extern*/ int Xdetab();
/*extern*/ int Xdisplay();
/*extern*/ int Xentab();
/*extern*/ int Xerrorclear();
/*extern*/ int Xexit();
/*extern*/ int Xfind();
/*extern*/ int Xget();
/*extern*/ int Xgetenv();
/*extern*/ int Xiand();
/*extern*/ int Xicom();
/*extern*/ int Ximage();
/*extern*/ int Xinsert();
/*extern*/ int Xinteger();
/*extern*/ int Xior();
/*extern*/ int Xishift();
/*extern*/ int Xixor();
/*extern*/ int Xleft();
/*extern*/ int Xlist();
/*extern*/ int Xmany();
/*extern*/ int Xmap();
/*extern*/ int Xmatch();
/*extern*/ int Xmember();
/*extern*/ int Xmove();
/*extern*/ int Xnumeric();
/*extern*/ int Xopen();
/*extern*/ int Xord();
/*extern*/ int Xpop();
/*extern*/ int Xpos();
/*extern*/ int Xproc();
/*extern*/ int Xpull();
/*extern*/ int Xpush();
/*extern*/ int Xput();
/*extern*/ int Xread();
/*extern*/ int Xreads();
/*extern*/ int Xreal();
/*extern*/ int Xremove();
/*extern*/ int Xrename();
/*extern*/ int Xrepl();
/*extern*/ int Xreverse();
/*extern*/ int Xright();
/*extern*/ int Xrunerr();
/*extern*/ int Xseek();
/*extern*/ int Xseq();
/*extern*/ int Xset();
/*extern*/ int Xsort();
/*extern*/ int Xstop();
/*extern*/ int Xstring();
/*extern*/ int Xsystem();
/*extern*/ int Xtab();
/*extern*/ int Xtable();
/*extern*/ int Xtrim();
/*extern*/ int Xtype();
/*extern*/ int Xupto();
/*extern*/ int Xwhere();
/*extern*/ int Xwrite();
/*extern*/ int Xwrites();
/*extern*/ novalue addmem();
/*extern*/ novalue adjust();
/*extern*/ struct astkblk *alcactiv();
/*extern*/ struct b_coexpr *alccoexp();
/*extern*/ struct b_cset *alccset();
/*extern*/ struct b_external *alcextrnl();
/*extern*/ struct b_file *alcfile();
/*extern*/ struct b_list *alclist();
/*extern*/ struct b_lelem *alclstb();
/*extern*/ struct b_real *alcreal();
/*extern*/ struct b_record *alcrecd();
/*extern*/ struct b_refresh *alcrefresh();
/*extern*/ struct b_selem *alcselem();
/*extern*/ struct b_set *alcset();
/*extern*/ char *alcstr();
/*extern*/ struct b_tvsubs *alcsubs();
/*extern*/ struct b_table *alctable();
/*extern*/ struct b_telem *alctelem();
/*extern*/ struct b_tvtbl *alctvtbl();
/*extern*/ int anycmp();
/*extern*/ int asgn();
/*extern*/ novalue atrace();
/*extern*/ int bang();
/*extern*/ int blkreq();
/*static*/ union block *alcblk();
/*extern*/ int bscan();
/*extern*/ novalue c_exit();
/*extern*/ int cat();
/*extern*/ novalue coacttrace();
/*extern*/ novalue cofailtrace();
/*extern*/ novalue cofree();
/*extern*/ novalue collect();
/*extern*/ novalue compact();
/*extern*/ int compl();
/*extern*/ novalue corettrace();
/*extern*/ int cplist();
/*extern*/ int create();
/*extern*/ int csetcmp();
/*static*/ int cstos();
/*extern*/ novalue ctrace();
/*extern*/ int cvcset();
/*extern*/ int cvint();
/*extern*/ int cvnum();
/*extern*/ long cvpos();
/*extern*/ int cvreal();
/*extern*/ int cvstr();
/*extern*/ novalue datainit();
/*extern*/ int defcset();
/*extern*/ int deffile();
/*extern*/ int defint();
/*extern*/ int defshort();
/*extern*/ int defstr();
/*extern*/ int deref();
/*extern*/ int diff();
/*extern*/ int divide();
/*extern*/ int doasgn();
/*extern*/ int doimage();
/*static*/ novalue env_err();
/*extern*/ novalue env_int();
/*extern*/ int envset();
/*extern*/ int equiv();
/*extern*/ int eqv();
/*extern*/ int err();
/*extern*/ novalue error();
/*extern*/ int escan();
/*extern*/ novalue fatalerr();
/*extern*/ int field();
/*extern*/ char *findfile();
/*extern*/ int findline();
/*extern*/ int fpetrap();
/*extern*/ novalue ftrace();
/*extern*/ int getstrg();
/*extern*/ long hash();
/*extern*/ char *icon_gcvt();
/*extern*/ novalue iconhost();
/*extern*/ novalue init();
/*extern*/ novalue initalloc();
/*extern*/ int inter();
/*extern*/ int interp();
/*extern*/ int invoke();
/*extern*/ long ipow();
/*static*/ int itos();
/*extern*/ int keywd();
/*extern*/ int lconcat();
/*extern*/ int lexcmp();
/*extern*/ int lexeq();
/*extern*/ int lexge();
/*extern*/ int lexgt();
/*extern*/ int lexle();
/*extern*/ int lexlt();
/*extern*/ int lexne();
/*extern*/ int limit();
/*static*/ novalue listimage();
/*extern*/ int llist();
/*extern*/ int locate();
/*extern*/ long longread();
/*extern*/ long lstrlen();
/*extern*/ novalue markblock();
/*extern*/ struct descrip *memb();
/*extern*/ struct descrip *memb_tbl();
/*extern*/ int minus();
/*extern*/ int mkreal();
/*extern*/ int mkrec();
/*extern*/ novalue mksubs();
/*extern*/ int mod();
/*extern*/ int mult();
/*extern*/ novalue mvc();
/*extern*/ int neg();
/*extern*/ int neqv();
/*extern*/ int nonnull();
/*extern*/ int null();
/*extern*/ int number();
/*extern*/ int numcmp();
/*extern*/ int numeq();
/*extern*/ int numge();
/*extern*/ int numgt();
/*extern*/ int numle();
/*extern*/ int numlt();
/*extern*/ int numne();
/*static*/ int nxttab();
/*extern*/ int order();
/*extern*/ novalue outimage();
/*extern*/ int plus();
/*extern*/ struct b_coexpr *popact();
/*extern*/ novalue postqual();
/*extern*/ int power();
/*extern*/ long prescan();
/*static*/ novalue printimage();
/*extern*/ int pushact();
/*extern*/ int puterr();
/*extern*/ int putpos();
/*extern*/ int putran();
/*extern*/ int putstr();
/*extern*/ int putsub();
/*extern*/ int puttrc();
/*extern*/ int qlcmp();
/*extern*/ int qtos();
/*static*/ int radix();
/*extern*/ int random();
/*extern*/ int rasgn();
/*static*/ novalue reclaim();
/*extern*/ int refresh();
/*extern*/ int resolve();
/*extern*/ int rswap();
/*extern*/ int rtos();
/*extern*/ novalue rtrace();
/*extern*/ novalue runerr();
/*extern*/ novalue scollect();
/*extern*/ int sect();
/*extern*/ int segvtrap();
/*static*/ novalue showlevel();
/*static*/ novalue showline();
/*extern*/ int size();
/*static*/ int stkdump();
/*static*/ int ston();
/*extern*/ novalue strace();
/*extern*/ int strprc();
/*extern*/ int strreq();
/*extern*/ int subsc();
/*extern*/ int swap();
/*extern*/ novalue sweep();
/*extern*/ novalue syserr();
/*extern*/ int tabmat();
/*extern*/ int toby();
/*extern*/ struct b_coexpr *topact();
/*extern*/ int trcmp3();
/*extern*/ int trefcmp();
/*extern*/ novalue ttrace();
/*extern*/ int tvalcmp();
/*extern*/ int tvcmp4();
/*extern*/ int unions();
/*extern*/ int value();
/*extern*/ novalue xdisp();
/*extern*/ novalue xtrace();

#endif					/* ProtoArgs */
