/*
 * Linker main program that controls the linking process.
 */

#include "../h/config.h"
#include "globals.h"
#include "link.h"
#include "general.h"
#include "../h/paths.h"
#include "../h/header.h"

#ifdef Header
#ifndef MaxHeader
#define MaxHeader MaxHdr
#endif					/* MaxHeader */

#ifndef HeaderFile
#define HeaderFile HeaderPath
#endif					/* HeaderFile */
#endif					/* Header */

/*
 * The following code is operating-system dependent [@link.01].  Include
 *  system-dependent files and declarations.
 */

#if PORT
   /* nothing to do */
#endif					/* PORT */

#if AMIGA || ATARI_ST || HIGHC_386 || MACINTOSH || MSDOS || VMS
   /* nothing to do */
#endif					/* AMIGA || ATARI_ST || HIGHC_386 ... */

#if UNIX
#include <sys/types.h>
#include <sys/stat.h>
#endif					/* UNIX */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */

FILE *infile;				/* input file (.u1 or .u2) */
FILE *outfile;				/* interpreter code output file */
FILE *dbgfile;				/* debug file */
char inname[MaxFileName];		/* input file name */
static char icnname[MaxFileName];	/* icon source file name */
static char dbgname[MaxFileName];	/* debug file name */

struct lfile *llfiles = NULL;		/* List of files to link */

int lineno = 0;				/* current source program line number */
static int fatals = 0;			/* number of errors encountered */

/*
 *  ilink - link a number of files, returning error count
 */
int ilink(ifiles,outname)
char **ifiles;
char *outname;
   {
   FILE *hfile;
   int i;
   struct lfile *lf,*lfls;
   char *filename;			/* name of current input file */

   linit();				/* initialize memory structures */
   while (*ifiles)
      alsolink (*ifiles++);		/* make initial list of files */

   /*
    * Phase I: load global information contained in .u2 files into
    *  data structures.
    *
    * The list of files to link is maintained as a queue with llfiles
    *  as the base.  lf moves along the list.  Each file is processed
    *  in turn by forming .u2 and .icn names from each file name, each
    *  of which ends in .u1.  The .u2 file is opened and globals is called
    *  to process it.  When the end of the list is reached, lf becomes
    *  NULL and the loop is terminated, completing phase I.  Note that
    *  link instructions in the .u2 file cause files to be added to list
    *  of files to link.
    */
   for (lf = llfiles; lf != NULL; lf = lf->lf_link) {
      filename = lf->lf_name;
      makename(inname, (char *) NULL, filename, ".u2");
      makename(icnname, "", filename, ".icn");
      infile = fopen(inname, "r");
      if (infile == NULL)
         quitf("cannot open %s",inname);
      readglob();
      fclose(infile);
      }

   /* Phase II: resolve undeclared variables and generate code. */

   /*
    * Open the output file.
    */
   outfile = fopen(outname, "wb");

/*
 * The following code is operating-system dependent [@link.02].  Set
 *  untranslated mode if necessary.
 */

#if PORT
   /* probably nothing */
#endif					/* PORT */

#if AMIGA || ATARI_ST || HIGHC_386 || MACINTOSH || UNIX || VMS
   /* nothing to do */
#endif					/* AMIGA || ATARI_ST || ... */

#if MSDOS
#if LATTICE
   fmode(outfile,1);			/* set for untranslated mode */
#endif					/* LATTICE */
#if MICROSOFT
   setmode(fileno(outfile),0x8000);	/* set for untranslated mode */
#endif					/* MICROSOFT */
#endif					/* MSDOS */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */
 
   if (outfile == NULL) 
      quitf("cannot create %s",outname);

#ifdef Header
   /*
    * Open Header, which contains the start-up program and copy it to the
    *  output file.  Then, write out null bytes to past the end of the
    *  start-up program.
    */
   {
   int hsize;
   char hname[MaxFileName];
   char hdrdat[MaxHeader+1];

/*
 * The following code is operating-system dependent [@link.03]. Open
 *  header file.
 */

#if PORT
   /* no header; irrelevant */
#endif					/* PORT */

#if AMIGA || ATARI_ST || HIGHC_386 || MSDOS || VMS
   /* no header; irrelevant */
#endif					/* AMIGA || ATARI_ST || HIGHC_386 ... */

#if MACINTOSH
#if MPW
      {  /* Look for header in same directory as the linker */
      char *p, *rindex();

      p = (char *)getenv("Command");
      if (p) {
         strcpy(hname,p);
         p = rindex(hname,':') + 1;
         }
      else
         p = hname;
      strcpy(p,HeaderFile);
      }
#endif					/* MPW */
#endif					/* MACINTOSH */

#if UNIX
   strcpy(hname,HeaderFile);
#endif					/* UNIX */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */

   hfile = fopen(hname,"rb");
   if (hfile == NULL)
      quitf("can't open linker header file %s",hname);
   hsize = fread(hdrdat, sizeof(char), MaxHeader + 1, hfile);
   if (hsize > MaxHeader)
      quitf("linker header file (%s) is too big", hname);
   fclose(hfile);
   fwrite(hdrdat, sizeof(char), hsize, outfile);
   for (i = MaxHeader - hsize + sizeof(struct header); i--;)
      putc(0, outfile);
   }
#else					/* Header */
   for (i = sizeof(struct header); i--;)
      putc(0, outfile);
#endif					/* Header */

   fflush(outfile);
   if (ferror(outfile) != 0)
      quit("unable to write to icode file");

   /*
    * Open the .ux file if debugging is on.
    */
   if (Dflag) {
      makename(dbgname, "", llfiles->lf_name, ".ux");
      dbgfile = fopen(dbgname, "w");
      if (dbgfile == NULL)
         quitf("cannot create %s", dbgname);
      }

   /*
    * Loop through input files and generate code for each.
    */
   lfls = llfiles;
   while (lf = getlfile(&lfls)) {
      filename = lf->lf_name;
      makename(inname, (char *) NULL, filename, ".u1");
      makename(icnname, "", filename, ".icn");
      infile = fopen(inname, "r");
      if (infile == NULL)
         quitf("cannot open %s", inname);
      gencode();
      fclose(infile);
      }
   gentables();		/* Generate record, field, global, global names,
			   static, and identifier tables. */
   fclose(outfile);
   if (fatals > 0)
      return fatals;
   setexe(outname);
   lmfree();
   return 0;
   }

/*
 * lwarn - issue a linker warning message.
 */
novalue lwarn(s1, s2, s3)
char *s1, *s2, *s3;
   {
   fprintf(stderr, "%s: ", icnname);
   if (lineno)
      fprintf(stderr, "%d: ", lineno);
   fprintf(stderr, "\"%s\": %s%s\n", s1, s2, s3);
   }

/*
 * lfatal - issue a fatal linker error message.
 */

novalue lfatal(s1, s2)
char *s1, *s2;
   {
   fprintf(stderr, "%s: ", icnname);
   if (lineno)
      fprintf(stderr, "%d: ", lineno);
   fprintf(stderr, "\"%s\": %s\n", s1, s2);
   fatals++;
   }

/*
 * setexe - mark the output file as executable
 */

static novalue setexe(fname)
char *fname;
   {

/*
 * The following code is operating-system dependent [@link.04].  It changes the
 *  mode of executable file so that it can be executed directly.
 */

#if PORT
   /* something is needed */
#endif					/* PORT */

#if AMIGA
   /* not necessary */
#endif					/* AMIGA */

#if ATARI_ST || MSDOS || VMS
   /* note: VMS files can't be made executable, but see "iexe.com" under VMS. */
#endif					/* ATARI_ST || MSDOS || VMS */

#if HIGHC_386
   /* not implemented yet. */
#endif					/* HIGHC_386 */

#if MACINTOSH
#if MPW
   printf("SetFile -t TEXT -c Icon %s\n",fname);
#endif					/* MPW */
#endif					/* MACINTOSH */

#if MSDOS
   /* probably could be smarter... */
   chmod(fname,0755);
#endif					/* MSDOS */

#if UNIX
      {
      struct stat stbuf;
      int u, r, m;
      /*
       * Set each of the three execute bits (owner,group,other) if allowed by
       *  the current umask and if the corresponding read bit is set; do not
       *  clear any bits already set.
       */
      umask(u = umask(0));		/* get and restore umask */
      if (stat(fname,&stbuf) == 0)  {	/* must first read existing mode */
         r = (stbuf.st_mode & 0444) >> 2;	/* get & position read bits */
         m = stbuf.st_mode | (r & ~u);		/* set execute bits */
         chmod(fname,m);		 /* change file mode */
         }
      }
#endif					/* UNIX */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */
   }
