/* tmain.c - main program for translator and linker */


#include <ctype.h>
#include "../h/config.h"
#include "../h/paths.h"
#include "general.h"

#ifdef strlen
#undef strlen				/* pre-defined in some contexts */
#endif					/* strlen */

#ifndef Iconx
#define Iconx IconxPath
#endif					/* Iconx */

/*
 *  Define global variables.
 */

#define Global
#define Init(v) = v
#include "globals.h"

/*
 * The following code is operating-system dependent [@tmain.01].
 *  -- include system-dependent files and declarations
 *  -- define the legal options and a corresponding usage message fragment
 *  -- define the extension used for icode files
 */

#if PORT
#define OPTIONS "ce:o:stuxLS:"
#define USAGE "[-cstu] [-e efile] [-o ofile] [-Sxnnnn]"
#define SUFFIX ""
#endif					/* PORT */

#if AMIGA
#include <dos.h>
#define OPTIONS "ce:o:stuxLS:"
#define USAGE "[-cstu] [-e efile] [-o ofile] [-Sxnnnn]"
#define SUFFIX ""
#endif					/* AMIGA */

#if ATARI_ST
#define OPTIONS "ce:o:p:stuxLS:"
#define USAGE "[-cstu] [-e efile] [-o ofile] [-p dir] [-Sxnnnn]"
#define SUFFIX ".icx"
#endif					/* ATARI_ST */

#if HIGHC_386
   /* not yet implemented */
#endif					/* HIGHC_386 */

#if MACINTOSH
#define OPTIONS "ce:o:stuxLS:"
#define USAGE "[-cstu] [-e efile] [-o ofile] [-Sxnnnn]"
#if MPW
#define SUFFIX ""
#endif					/* MPW */
#if LSC
#define SUFFIX ".icx"
#endif					/* LSC */
#endif					/* MACINTOSH */

#if MSDOS
#define OPTIONS "ce:o:stuxLS:"
#define USAGE "[-cstu] [-e efile] [-o ofile] [-Sxnnnn]"
#define SUFFIX ".icx"
#endif					/* MSDOS */

#if UNIX
#define OPTIONS "ce:mo:stuxLS:"
#define USAGE "[-cmstu] [-e efile] [-o ofile] [-Sxnnnn]"
#define SUFFIX ""
#endif					/* UNIX */

#if VMS
#define OPTIONS "ce:o:stuxLS:"
#define USAGE "[-cstu] [-e efile] [-o ofile] [\"-Sxnnnn\"]"
#define SUFFIX ".icx"
#endif					/* VMS */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */

/*
 * getopt() variables
 */
static int optind = 1;		/* index into parent argv vector */
static int optopt;		/* character checked for validity */
static char *optarg;		/* argument associated with option */

/*
 *  main program
 */
novalue main(argc,argv)
int argc;
char **argv;
   {
   int nolink = 0;			/* suppress linking? */
   int errors = 0;			/* translator and linker errors */
   char **tfiles, **tptr;		/* list of files to translate */
   char **lfiles, **lptr;		/* list of files to link */
   char **rfiles, **rptr;		/* list of files to remove */
   char *efile = NULL;			/* stderr file */
   char *ofile = NULL;			/* linker output file name */
   char buf[MaxFileName];		/* file name construction buffer */
   int c, n;
   struct fileparts fp;

   /*
    * Process options.
    */
   while ((c = getopt(argc,argv,OPTIONS)) != EOF)
      switch (c) {
         case 'c':			/* -c: compile only (no linking) */
            nolink = 1;
            break;
         case 'e':			/* -e file: redirect stderr */
            efile = optarg;
            break;
         case 'm':			/* -m: preprocess using m4(1) [UNIX] */
            m4pre = 1;
            break;
         case 'o':			/* -o file: name output file */
            ofile = optarg;
            break;
         case 'p':			/* -p path: iconx path [ATARI] */
            patharg = optarg;
            break;
         case 's':			/* -s: suppress informative messages */
            silent = 1;
            break;
         case 'u':			/* -u: warn about undeclared ids */
            uwarn = 1;
            break;
         case 't':			/* -t: turn on tracing */
            trace = -1;
            break;
         case 'L':			/* -L: enable linker debugging */
            Dflag = 1;
            break;
         case 'S':			/* -Sxnnnn: set a size */
            sizearg(optarg,argv);
            break;
         default:
         case 'x':			/* -x illegal until after file list */
            usage();
         }

   /*
    * Allocate space for lists of file names.
    */
   n = argc - optind + 1;
   tptr = tfiles = (char **)alloc((unsigned int)(n * sizeof(char *)));
   lptr = lfiles = (char **)alloc((unsigned int)(n * sizeof(char *)));
   rptr = rfiles = (char **)alloc((unsigned int)(2 * n * sizeof(char *)));

   /*
    * Scan file name arguments.
    */
   while (optind < argc)  {
      if (strcmp(argv[optind],"-x") == 0)	/* stop at -x */
         break;
      else if (strcmp(argv[optind],"-") == 0) {
         *tptr++ = "-";				/* "-" means standard input */
         *lptr++ = *rptr++ = "stdin.u1";
         *rptr++ = "stdin.u2";
         }
      else {
         fp = fparse(argv[optind]);		/* parse file name */
         if (*fp.ext == '\0' || smatch(fp.ext,".icn")) {
            makename(buf,(char*)NULL,argv[optind],".icn");
            *tptr++ = salloc(buf);		/* translate the .icn file */
            makename(buf,"",argv[optind],".u1");
            *lptr++ = *rptr++ = salloc(buf);	/* link & remove .u1 */
            buf[strlen(buf)-1] = '2';
            *rptr++ = salloc(buf);		/* also remove .u2 */
            }
         else if (smatch(fp.ext,".u1") || smatch(fp.ext,".u2")
               || smatch(fp.ext,".u")) {
            makename(buf,(char*)NULL,argv[optind],".u1");
            *lptr++ = salloc(buf);
            }
         else
            quitf("bad argument %s",argv[optind]);
         }
      optind++;
      }

   *tptr = *lptr = *rptr = NULL;	/* terminate filename lists */
   if (lptr == lfiles)
      usage();				/* error -- no files named */

   /*
    * Round hash table sizes to next power of two, and set masks for hashing.
    */
   chsize = round2(chsize);  cmask = chsize - 1;
   fhsize = round2(fhsize);  fmask = fhsize - 1;
   ghsize = round2(ghsize);  gmask = ghsize - 1;
   ihsize = round2(ihsize);  imask = ihsize - 1;
   lhsize = round2(lhsize);  lmask = lhsize - 1;

   /*
    * Translate .icn files to make .u1 and .u2 files.
    */
   if (tptr > tfiles) {
      if (!silent)
         report("Translating");
      errors = trans(tfiles);
      if (errors > 0)			/* exit if errors seen */
         exit(ErrorExit);
      }

   /*
    * Link .u1 and .u2 files to make an executable.
    */
   if (nolink)				/* exit if no linking wanted */
      exit(NormalExit);

   if (ofile == NULL)  {		/* if no -o file, synthesize a name */
      ofile = salloc(makename(buf,"",lfiles[0],SUFFIX));
   } else {				/* add extension in necessary */
      fp = fparse(ofile);
      if (*fp.ext == '\0' && *SUFFIX != '\0') /* if no ext given, but wanted */
         ofile = salloc(makename(buf,(char*)NULL,ofile,SUFFIX));
   }

   if (!silent)
      report("Linking");
   errors = ilink(lfiles,ofile);	/* link .u files to make icode file */

   /*
    * Finish by removing intermediate files.
    *  Execute the linked program if so requested and if there were no errors.
    */
   rmfiles(rfiles);			/* remove intermediate files */
   if (errors > 0)			/* exit if linker errors seen */
      exit(ErrorExit);

   if (optind < argc)  {
      if (!silent)
         report("Executing");
      execute (ofile, efile, argv+optind+1);
      }

   exit(NormalExit);
   }

/*
 * execute - execute iconx to run the icon program
 */
static novalue execute(ofile,efile,args)
char *ofile, *efile, **args;
   {
   int n;
   char **argv, **p;

   for (n = 0; args[n] != NULL; n++)	/* count arguments */
      ;
   p = argv = (char **)alloc((unsigned int)((n + 5) * sizeof(char *)));
   *p++ = Iconx;			/* set iconx pathname */
   if (efile != NULL) {			/* if -e given, copy it */
      *p++ = "-e";
      *p++ = efile;
      }
   *p++ = ofile;			/* pass icode file name */
   while (*p++ = *args++)		/* copy args into argument vector */
      ;
   *p = NULL;

/*
 * The following code is operating-system dependent [@tmain.02].  It calls
 *  iconx on the way out.
 */

#if PORT
   /* something is needed */
#endif					/* PORT */

#if AMIGA
      {
      struct ProcID procid;
      if (forkv(Iconx,argv,NULL,&procid) == 0) { 
         wait(&procid);
         return;
      }
#endif					/* AMIGA */

#if ATARI_ST
      Forkvp(Iconx,argv);	/* execute with path search */
      return;
#endif					/* ATARI_ST */

#if HIGHC_386
  /* not implemented yet */
#endif					/* HIGHC_386 */

#if MACINTOSH
#if MPW
   fputs(Iconx,stdout);
   for (p = argv + 1; *p; ++p) {
      putchar(' ');
      fputs(*p,stdout);
      }
   putchar('\n');
   return;
#endif					/* MPW */
#endif					/* MACINTOSH */

#if MSDOS
      execvp(Iconx,argv);	/* execute with path search */
#endif					/* MSDOS */

#if UNIX || VMS
      execv(Iconx,argv);
#endif					/* UNIX || VMS */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */

   quitf("could not run %s",Iconx);
   }

/*
 * get command option.  Based on a public domain implementation of System V
 *  getopt(3) by Keith Bostic (keith@seismo), Aug 24, 1984.
 */

#define BADCH	(int)'?'
#define EMSG	""
#define tell(m)	fprintf(stderr,"%s: %s -- %c\n",progname,m,optopt);return BADCH;

static int getopt(nargc,nargv,ostr)
int nargc;
char **nargv, *ostr;
   {
   static char *place = EMSG;		/* option letter processing */
   register char *oli;			/* option letter list index */
   char *index();

   if(!*place) {			/* update scanning pointer */
      if(optind >= nargc || *(place = nargv[optind]) != '-' || !*++place)
         return(EOF);
      if (*place == '-') {		/* found "--" */
         ++optind;
         return(EOF);
         }
      }					/* option letter okay? */
   if ((optopt = (int)*place++) == (int)':' || !(oli = index(ostr,optopt))) {
      if(!*place) ++optind;
      tell("illegal option");
      }
   if (*++oli != ':') {			/* don't need argument */
      optarg = NULL;
      if (!*place) ++optind;
      }
   else {				/* need an argument */
      if (*place) optarg = place;	/* no white space */
      else if (nargc <= ++optind) {	/* no arg */
         place = EMSG;
         tell("option requires an argument");
         }
      else optarg = nargv[optind];	/* white space */
      place = EMSG;
      ++optind;
      }
   return(optopt);			/* dump back option letter */
   }

static novalue report(s)
char *s;
   {

/*
 * The following code is operating-system dependent [@tmain.03].  Report
 *  phase.
 */

#if PORT
   fprintf(stderr,"%s:\n",s);
#endif					/* PORT */

#if AMIGA || ATARI_ST || HIGHC_386 || MSDOS || UNIX || VMS
   fprintf(stderr,"%s:\n",s);
#endif					/* AMIGA || ATARI_ST || HIGHC_386 ... */

#if MACINTOSH
#if MPW
   printf("Echo %s:\n",s);
#endif					/* MPW */
#if LSC
   fprintf(stderr,"%s:\n",s);
#endif					/* LSC */
#endif					/* MACINTOSH */

#if VM || MVS
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */

   }

/*
 * rmfiles - remove a list of files
 */

static novalue rmfiles(p)
char **p;
   {
   for (; *p; p++) {
      remove(*p);
      }
   }

/*
 * round2 - round an integer up to the next power of 2.
 */
static unsigned int round2(n)
unsigned int n;
   {
   unsigned int b = 1;
   while (b < n)
      b <<= 1;
   return b;
   }

/*
 * sizearg - process -S command option.
 */

struct keyptr {			/* structure for listing option chars */
   char *cmd;				/* option character(s) */
   unsigned int *valp;			/* pointer to value word */
   };

static struct keyptr keytable[] = {	/* maps keys to store addresses */
#define Size(cmd,vname,defalt) cmd, &vname,
#include "sizes.h"			/* initialize from "sizes.h" data */
#undef Size
   0, 0,				/* terminate with null entry */
   };

static novalue sizearg(arg,argv)
char *arg;
char **argv;
   {
   struct keyptr *k;			/* key table pointer */
   char *s;				/* value string pointer */
   int v;				/* option value */

   for (k = keytable; k->cmd; k++)	/* search for key match */
      if (arg[0] == k->cmd[0] && (arg[0] != 'h' || arg[1] == k->cmd[1]))
         break;

   if (k->cmd == NULL)			/* abort if not found */
      quitf("unrecognized -S option: -S%s",arg);

   if (arg[0] == 'h')
      s = &arg[2];			/* find value */
   else
      s = &arg[1];
      
   if (*s == '\0') {			/* if value is in next arg */
      s = argv[optind++];
      if (s == NULL)
         quitf("missing value: -S%s", arg);
      }

   v = (int)atol(s);				/* convert integer -- check */
   if (v <= 0)
      quitf("illegal value: -S%s", arg);
   *k->valp = v;			/* store result */
   }

/*
 * smatch - case-insensitive string match - returns nonzero if they match
 */
static int smatch(s,t)
char *s, *t;
   {
   char a, b;
   for (;;) {
      while (*s == *t)
         if (*s++ == '\0')
            return 1;
         else
            t++;
      a = *s++;
      b = *t++;
      if (isupper(a))  a = tolower(a);
      if (isupper(b))  b = tolower(b);
      if (a != b)
         return 0;
      }
   }

/*
 * Print an error message if called incorrectly.  The message depends
 *  on the legal options for this system.
 */
static novalue usage()
   {
   fprintf(stderr,"usage: %s %s file ... [-x args]\n", progname, USAGE);
   exit(ErrorExit);
   }
