/*
 *  General utility routines.
 */

#include "../h/config.h"
#include "../h/cpuconf.h"
#include "general.h"
#include "trans.h"
#include "tree.h"

/*
 * The following code is operating-system dependent [@util.01].  Define the
 *  characters that terminate a file name prefix.
 */

#if PORT
#define PREFIX "/"
#endif					/* PORT */

#if AMIGA
#define PREFIX "/:"
#endif					/* AMIGA */

#if ATARI_ST
#define PREFIX "/:\\"
#endif					/* ATARI_ST */

#if HIGHC_386 || MSDOS
#define PREFIX "/:\\"
#endif					/* HIGHC_386 || MSDOS*/

#if MACINTOSH
#if MPW || LSC
#define PREFIX ":"
#endif					/* MPW || LSC */
#endif					/* MACINTOSH */

#if UNIX
#define PREFIX "/"
#endif					/* UNIX */

#if VMS
#define PREFIX "]:"
#endif					/* VMS */

#if VM || MVS
#define PREFIX ""
#endif					/* VM || MVS */

/*
 * End of operating-system specific code.
 */

/*
 * alloc - allocate n bytes
 */

pointer alloc(n)
unsigned int n;
   {
   pointer a, malloc();

   if (!(a = malloc(n)))
      quit("out of memory");
   return a;
   }

/*
 * salloc - allocate and initialize string 
 */

char *salloc(s)
char *s;
   {
   char *strcpy();

   return strcpy((char *)alloc((unsigned int)(strlen(s)+1)),s);
   }

/*
 * tcalloc - allocate and zero m*n bytes
 */
pointer tcalloc(m,n)
unsigned int m, n;
   {
   pointer a, calloc();

   if (!(a = calloc(m,n)))
      quit("out of memory");
   return a;
   }

/*
 * fparse - break a file name down into component parts.
 *  Result is a struct of static pointers good until the next call.
 */
struct fileparts fparse(s)
char *s;
   {
   static char buf[MaxFileName+2];
   int n;
   char *p, *q;
   struct fileparts fp;

   q = s;
   fp.ext = p = s + strlen(s);
   while (--p >= s) {
      if (*p == '.' && *fp.ext == '\0')
         fp.ext = p;
      else if (index(PREFIX,*p)) {
         q = p+1;
         break;
         }
      }
   fp.dir = buf;
   n = q - s;
   strncpy(fp.dir,s,n);
   fp.dir[n] = '\0';
   fp.name = buf + n + 1;
   n = fp.ext - q;
   strncpy(fp.name,q,n);
   fp.name[n] = '\0';
   return fp;
   }

/*
 * makename - make a file name, optionally substituting a new dir and/or ext
 */
char *makename(dest,d,name,e)
char *dest, *d, *name, *e;
   {
   struct fileparts fp;
   fp = fparse(name);
   if (d != NULL)
      fp.dir = d;
   if (e != NULL)
      fp.ext = e;
   sprintf(dest,"%s%s%s",fp.dir,fp.name,fp.ext);
   return dest;
   }

/*
 * quit - immediate exit with error message
 */

novalue quit(msg)
char *msg;
   {
   quitf(msg,"");
   }

/*
 * quitf - immediate exit with message format and argument
 */
novalue quitf(msg,arg)
char *msg, *arg;
   {
   extern char *progname;
   fprintf(stderr,"%s: ",progname);
   fprintf(stderr,msg,arg);
   fprintf(stderr,"\n");
   exit(ErrorExit);
   }

#if IntBits == 16
/*
 * Long strlen
 */

long lstrlen(s)
char *s;
{
    long l = 0;
    while(*s++) l++;
    return l;
}
#endif					/* INtBits */

/* 
 * Write a long string in int-sized chunks.
 */

novalue longwrite(s,len,file)
FILE *file;
char *s;
long len;
{
   long tally, n;
   int leftover, loopnum;
   char *p;

   tally = 0;
   loopnum = len / MaxInt;
   leftover = len % MaxInt;
   for (p = s, loopnum = len/MaxInt; loopnum; loopnum--) {
       n = fwrite(p,sizeof(char),MaxInt,file);
       tally += n;
       p += MaxInt;
   }
   if (leftover)
      n = fwrite(p,sizeof(char),leftover,file);
   tally += n;
   if (tally != len)
      quit("unable to write to file");
   }

/*
 * tsyserr is called for fatal errors.  The message s is produced and the
 *  translator exits.
 */
novalue tsyserr(s)
char *s;
   {
   if (tok_loc.n_file)
      fprintf(stderr, "File %s; ", tok_loc.n_file);
   fprintf(stderr, "Line %d # %s\n", inline, s);
   exit(ErrorExit);
   }
